package lockmgr;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class UpgradableLock{
	
	private final ReentrantReadWriteLock lock = new ReentrantReadWriteLock(true);
	
	public synchronized boolean tryReadLock(){		
		return lock.readLock().tryLock();		
	}
	
	public synchronized boolean readLock(){
		if(lock.getReadHoldCount() > 0)
			return true;
		
				
			try {
				while(!lock.readLock().tryLock(0, TimeUnit.NANOSECONDS)){
					this.wait();
				}
			} catch (InterruptedException e) {
				//must not happen
				e.printStackTrace();
				return false;
			}
		return true;
	}
	
	public synchronized boolean tryWriteLock(){		
		return lock.writeLock().tryLock();		
	}
	
	public synchronized boolean writeLock(){
		if(lock.getWriteHoldCount() > 0)
			return true;
	
		try {				
			while(!lock.writeLock().tryLock(0, TimeUnit.NANOSECONDS)){
				this.wait();
			}		
		} catch (InterruptedException e) {
			//must not happen
			e.printStackTrace();
			return false;
		}

		return true;
	}
	
	public boolean holdsReadlock(){
		return lock.getReadHoldCount() > 0;
	}
	
	public boolean holdsWritelock(){
		return lock.getWriteHoldCount() > 0;
	}
	
	public synchronized boolean upgradeLock(){
		
		if(lock.getReadHoldCount() == 0)
			return false;
		
		try {
			
			lock.readLock().unlock();
			while(!lock.writeLock().tryLock()){
				this.wait();
			}		
		} catch (InterruptedException e) {
			//must not happen
			e.printStackTrace();
			return false;
		}		
		return true;
	}
	
	public synchronized void unlock(){	
		for(int i=0;i<lock.getReadHoldCount();i++)
			lock.readLock().unlock();
		for(int i=0;i<lock.getWriteHoldCount();i++)
			lock.writeLock().unlock();
		
		this.notifyAll();		
	}

}
