package transaction;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import transaction.LogEntry.LogType;


/**
 * A simple log manager which logs operations on a ResourceManager
 * in memory.
 * @author bross
 *
 */
public class LogManager {
	
	private Hashtable<Integer, LinkedList<LogEntry>> log;
	
	
	public LogManager(){
		log = new Hashtable<Integer, LinkedList<LogEntry>>();
	}
	
	public boolean log(int xid, String resourceID, LogType type,
			ResourceData afterImage, String reservation){
		
		LogEntry u = new LogEntry(xid, resourceID, type, afterImage, reservation);
		LinkedList<LogEntry> logEntries = log.get(xid);
		if(logEntries == null){
			logEntries = new LinkedList<LogEntry>();
			log.put(xid, logEntries);
		}
				
		logEntries.add(u);
		
		return true;
	}		
	
	public LinkedList<LogEntry> getLogEntries(int xid){
		return log.get(xid);
	}
	
	public void deleteLogsForTransaction(int xid){
		log.remove(xid);		
	}	
	
	public ResourceData insertedResource(int xid, String rid){
		LinkedList<LogEntry> logEntries = log.get(xid);
		if(logEntries != null){
			for (LogEntry logEntry : logEntries) {
				if(logEntry.resourceID.equals(rid) && logEntry.type == LogType.INSERT_RESOURCE)
					return logEntry.afterImage;
			}
		}
		
		return null;
	}		
	
	public boolean insertedCustomer(int xid, String rid){
		LinkedList<LogEntry> logEntries = log.get(xid);
		if(logEntries != null){
			for (LogEntry logEntry : logEntries) {
				if(logEntry.resourceID.equals(rid) && logEntry.type == LogType.INSERT_CUSTOMER)
					return true;
			}
		}
		
		return false;
	}		
	
	public Set<String> getReservations(int xid, String customer){
		Set<String> reservations = new HashSet<String>();
		LinkedList<LogEntry> logEntries = log.get(xid);
		if(logEntries != null){
			for (LogEntry logEntry : logEntries) {
				if(logEntry.resourceID.equals(customer) && logEntry.type == LogType.INSERT_RESERVATION)
					reservations.add(logEntry.reservation);
			}
		}
		
		return reservations;		
	}
	
	public ResourceData getLatestUpdate(int xid, String rid){
		LinkedList<LogEntry> logEntries = log.get(xid);
		LogEntry entry = null;
		if(logEntries != null){
			Iterator<LogEntry> it = logEntries.descendingIterator();
			while(it.hasNext()){
				entry = it.next();
				if(entry.type == LogType.UPDATE_RESOURCE && entry.xid == xid && entry.resourceID.equals(rid))
					return entry.afterImage;
			}
		}
		
		return null;
	}
	
	public Map<String, ResourceData> getLatestResourceUpdates(int xid){
		HashMap<String, ResourceData> updates = new HashMap<String, ResourceData>();
		LinkedList<LogEntry> logEntries = log.get(xid);
		if(logEntries != null){
			for (LogEntry logEntry : logEntries) {
				if(logEntry.type == LogType.INSERT_RESOURCE || logEntry.type == LogType.UPDATE_RESOURCE)
					updates.put(logEntry.resourceID, logEntry.afterImage);
			}
		}
		
		return updates;
		
	}
	

}
